@@ -9,7 +9,7 @@ use crate::{Error, PolygonScalar};
 //    No panics.
 //    All Ok results are valid convex polygons.
 //    No points are outside the resulting convex polygon.
-/// $O(n log n)$ Convex hull of a set of points.
+/// $O(n \log n)$ Convex hull of a set of points.
 ///
 /// [Graham scan][wiki] algorithm for finding the smallest convex polygon which
 /// contains all the given points.
@@ -87,12 +87,13 @@ where
   pts.sort_unstable_by(|a, b| {
     smallest
       .ccw_cmp_around(a, b)
-      .then_with(|| smallest.cmp_distance_to(a, b))
+      .then_with(|| (a.y_coord(), a.x_coord()).cmp(&(b.y_coord(), b.x_coord())))
+    // .then_with(|| smallest.cmp_distance_to(a, b))
   });
-  pts.dedup();
   if pts.len() < 3 {
     return Err(Error::InsufficientVertices);
   }
+  debug_assert_eq!(&pts[0], smallest);
   let mut write_idx = 1;
   let mut read_idx = 2;
   // Drop points that are co-linear with our origin.
@@ -195,6 +196,35 @@ mod tests {
     assert_ok!(poly.validate());
   }
 
+  #[test]
+  fn convex_hull_dups() {
+    let points = vec![
+      Point::new([0, 0]),
+      Point::new([1, 0]),
+      Point::new([0, 0]),
+      Point::new([1, 0]),
+      Point::new([2, 2]),
+      Point::new([2, 2]),
+      Point::new([5, 1]),
+      Point::new([5, 1]),
+    ];
+    let poly = convex_hull(points).unwrap();
+    assert_ok!(poly.validate());
+  }
+
+  #[test]
+  fn convex_hull_insufficient_dups() {
+    let points = vec![
+      Point::new([0, 0]),
+      Point::new([0, 0]),
+      Point::new([2, 2]),
+      Point::new([2, 2]),
+      Point::new([0, 0]),
+      Point::new([2, 2]),
+    ];
+    assert_eq!(convex_hull(points).err(), Some(Error::InsufficientVertices));
+  }
+
   #[test]
   fn convex_hull_invalid() {
     let points: Vec<Point<i64, 2>> = vec![
@@ -210,19 +240,58 @@ mod tests {
     assert_ok!(poly.validate());
   }
 
+  #[test]
+  fn unit_1() {
+    let points: Vec<Point<BigInt, 2>> = vec![
+      Point::new([0, 0]).into(),
+      Point::new([-1, 1]).into(),
+      Point::new([0, 1]).into(),
+      Point::new([-717193444810564826, 1]).into(),
+    ];
+    let poly = convex_hull(points).unwrap();
+    assert_ok!(poly.validate());
+  }
+
+  #[test]
+  fn unit_2() {
+    let points: Vec<Point<i8, 2>> = vec![
+      Point::new([0, 0]),
+      Point::new([0, -10]),
+      Point::new([-13, 0]),
+    ];
+    let poly = convex_hull(points).unwrap();
+    assert_ok!(poly.validate());
+  }
+
   proptest! {
     #[test]
     fn convex_hull_prop(pts in vec(any_r(), 0..100)) {
       if let Ok(poly) = convex_hull(pts.clone()) {
         // Prop #1: Results are valid.
-        assert_ok!(poly.validate());
+        prop_assert_eq!(poly.validate().err(), None);
         // Prop #2: No points from the input set are outside the polygon.
         for pt in pts.iter() {
-          assert_ne!(poly.locate(pt), PointLocation::Outside)
+          prop_assert_ne!(poly.locate(pt), PointLocation::Outside)
         }
         // Prop #3: All vertices are in the input set.
-        for (pt, _meta) in poly.iter() {
-          assert!(pts.contains(pt))
+        for pt in poly.iter() {
+          prop_assert!(pts.contains(pt))
+        }
+      }
+    }
+
+    #[test]
+    fn convex_hull_prop_i8(pts in vec(any_8(), 0..100)) {
+      if let Ok(poly) = convex_hull(pts.clone()) {
+        // Prop #1: Results are valid.
+        prop_assert_eq!(poly.validate().err(), None);
+        // Prop #2: No points from the input set are outside the polygon.
+        for pt in pts.iter() {
+          prop_assert_ne!(poly.locate(pt), PointLocation::Outside)
+        }
+        // Prop #3: All vertices are in the input set.
+        for pt in poly.iter() {
+          prop_assert!(pts.contains(pt))
         }
       }
     }
