@@ -8,6 +8,7 @@ use rand::Rng;
 use std::cmp::Ordering;
 use std::convert::TryFrom;
 use std::fmt::Debug;
+use std::ops::Deref;
 use std::ops::Index;
 use std::ops::Neg;
 
@@ -34,8 +35,8 @@ where
 }
 
 // Methods on N-dimensional points.
-impl<T: Clone, const N: usize> Point<T, N> {
-  pub fn new(array: [T; N]) -> Point<T, N> {
+impl<T, const N: usize> Point<T, N> {
+  pub const fn new(array: [T; N]) -> Point<T, N> {
     Point { array }
   }
 
@@ -53,29 +54,30 @@ impl<T: Clone, const N: usize> Point<T, N> {
     self.into()
   }
 
+  // Warning: May cause arithmetic overflow.
   pub fn cmp_distance_to(&self, p: &Point<T, N>, q: &Point<T, N>) -> Ordering
   where
-    T: Clone + Zero + Ord + NumOps,
-    // for<'a> &'a T: Mul<&'a T, Output = T> + Sub<&'a T, Output = T>,
+    T: Clone + Zero + Ord + NumOps + crate::Extended,
   {
     self
       .squared_euclidean_distance(p)
       .cmp(&self.squared_euclidean_distance(q))
   }
 
-  pub fn squared_euclidean_distance(&self, rhs: &Point<T, N>) -> T
+  // Warning: May cause arithmetic overflow.
+  pub fn squared_euclidean_distance(&self, rhs: &Point<T, N>) -> T::ExtendedSigned
   where
-    T: Clone + Zero + NumOps,
-    // for<'a> &'a T: Mul<&'a T, Output = T> + Sub<&'a T, Output = T>,
+    T: Clone + Zero + NumOps + crate::Extended,
   {
     self
       .array
       .iter()
       .zip(rhs.array.iter())
-      .fold(T::zero(), |sum, (a, b)| {
-        let diff: T = a.clone() - b.clone();
-        sum + diff.clone() * diff
+      .map(|(a, b)| {
+        let diff = a.clone().extend_signed() - b.clone().extend_signed();
+        diff.clone() * diff
       })
+      .sum()
   }
 
   // Similar to num_traits::identities::Zero but doesn't require an Add impl.
@@ -90,6 +92,7 @@ impl<T: Clone, const N: usize> Point<T, N> {
 
   pub fn cast<U, F>(&self, f: F) -> Point<U, N>
   where
+    T: Clone,
     F: Fn(T) -> U,
   {
     Point {
@@ -114,12 +117,19 @@ impl<'a, const N: usize> TryFrom<Point<f64, N>> for Point<NotNan<f64>, N> {
   }
 }
 
-impl<'a, const N: usize> TryFrom<Point<f64, N>> for Point<BigRational, N> {
-  type Error = ();
-  fn try_from(point: Point<f64, N>) -> Result<Point<BigRational, N>, ()> {
-    Ok(Point {
-      array: try_array_init(|i| BigRational::from_float(point.array[i]).ok_or(()))?,
-    })
+impl<T> From<(T, T)> for Point<T, 2> {
+  fn from(point: (T, T)) -> Point<T, 2> {
+    Point {
+      array: [point.0, point.1],
+    }
+  }
+}
+
+impl From<Point<i64, 2>> for Point<BigInt, 2> {
+  fn from(point: Point<i64, 2>) -> Point<BigInt, 2> {
+    Point {
+      array: [point.array[0].into(), point.array[1].into()],
+    }
   }
 }
 
@@ -131,6 +141,14 @@ impl<'a, const N: usize> From<&'a Point<BigRational, N>> for Point<f64, N> {
   }
 }
 
+impl<const N: usize> From<Point<BigRational, N>> for Point<f64, N> {
+  fn from(point: Point<BigRational, N>) -> Point<f64, N> {
+    Point {
+      array: array_init(|i| point.array[i].to_f64().unwrap()),
+    }
+  }
+}
+
 impl<'a, const N: usize> From<&'a Point<f64, N>> for Point<BigRational, N> {
   fn from(point: &Point<f64, N>) -> Point<BigRational, N> {
     Point {
@@ -139,6 +157,14 @@ impl<'a, const N: usize> From<&'a Point<f64, N>> for Point<BigRational, N> {
   }
 }
 
+impl<const N: usize> From<Point<f64, N>> for Point<BigRational, N> {
+  fn from(point: Point<f64, N>) -> Point<BigRational, N> {
+    Point {
+      array: array_init(|i| BigRational::from_f64(point.array[i]).unwrap()),
+    }
+  }
+}
+
 impl<T, const N: usize> From<Vector<T, N>> for Point<T, N> {
   fn from(vector: Vector<T, N>) -> Point<T, N> {
     Point { array: vector.0 }
@@ -163,16 +189,15 @@ impl<T, const N: usize> From<Vector<T, N>> for Point<T, N> {
 impl<T> Point<T, 2> {
   pub fn orientation(&self, q: &Point<T, 2>, r: &Point<T, 2>) -> Orientation
   where
-    T: Clone + NumOps + Ord,
-    // for<'a> &'a T: Sub<Output = T>,
+    T: Clone + NumOps + Ord + crate::Extended,
   {
-    raw_arr_turn(&self.array, &q.array, &r.array)
+    Orientation::new(&self.array, &q.array, &r.array)
   }
 
   /// Docs?
   pub fn ccw_cmp_around(&self, p: &Point<T, 2>, q: &Point<T, 2>) -> Ordering
   where
-    T: Clone + Ord + NumOps + Zero + One + Neg<Output = T>,
+    T: Clone + Ord + NumOps + Zero + One + Neg<Output = T> + crate::Extended + Signed,
     // for<'a> &'a T: Mul<&'a T, Output = T>,
   {
     self.ccw_cmp_around_with(&Vector([T::one(), T::zero()]), p, q)
@@ -180,10 +205,10 @@ impl<T> Point<T, 2> {
 
   pub fn ccw_cmp_around_with(&self, z: &Vector<T, 2>, p: &Point<T, 2>, q: &Point<T, 2>) -> Ordering
   where
-    T: Clone + Ord + NumOps + Neg<Output = T>,
+    T: Clone + Ord + NumOps + Neg<Output = T> + crate::Extended + Signed,
     // for<'a> &'a T: Mul<Output = T>,
   {
-    ccw_cmp_around_origin_with(&z.0, &(p - self).0, &(q - self).0)
+    ccw_cmp_around_with(&z.0, &self, &p, &q)
   }
 }
 
@@ -213,6 +238,13 @@ impl<T> Point<T, 3> {
   }
 }
 
+impl<T, const N: usize> Deref for Point<T, N> {
+  type Target = [T; N];
+  fn deref(&self) -> &[T; N] {
+    &self.array
+  }
+}
+
 mod add;
 mod sub;
 
@@ -329,6 +361,26 @@ pub mod tests {
     }
   }
 
+  impl<const N: usize> Arbitrary for Point<i64, N> {
+    type Strategy = Point<num::i64::Any, N>;
+    type Parameters = ();
+    fn arbitrary_with(_params: ()) -> Self::Strategy {
+      Point {
+        array: array_init(|_| any::<i64>()),
+      }
+    }
+  }
+
+  impl<const N: usize> Arbitrary for Point<i8, N> {
+    type Strategy = Point<num::i8::Any, N>;
+    type Parameters = ();
+    fn arbitrary_with(_params: ()) -> Self::Strategy {
+      Point {
+        array: array_init(|_| any::<i8>()),
+      }
+    }
+  }
+
   pub fn any_nn<const N: usize>() -> impl Strategy<Value = Point<NotNan<f64>, N>> {
     any::<Point<f64, N>>().prop_filter_map("Check for NaN", |pt| pt.try_into().ok())
   }
@@ -337,26 +389,68 @@ pub mod tests {
     any::<Point<isize, N>>().prop_map(|pt| pt.cast(BigInt::from))
   }
 
+  pub fn any_64<const N: usize>() -> impl Strategy<Value = Point<i64, N>> {
+    any::<Point<i64, N>>()
+  }
+
+  pub fn any_8<const N: usize>() -> impl Strategy<Value = Point<i8, N>> {
+    any::<Point<i8, N>>()
+  }
+
   proptest! {
     #[test]
-    fn squared_euclidean_distance_fuzz(pt1: Point<f64,2>, pt2: Point<f64,2>) {
+    fn squared_euclidean_distance_fuzz(pt1 in any_nn::<2>(), pt2 in any_nn::<2>()) {
       let _ = pt1.squared_euclidean_distance(&pt2);
     }
 
     #[test]
-    fn cmp_around_fuzz(pt1 in any_nn(), pt2 in any_nn(), pt3 in any_nn()) {
+    fn cmp_around_fuzz_nn(pt1 in any_nn(), pt2 in any_nn(), pt3 in any_nn()) {
       let _ = pt1.ccw_cmp_around(&pt2, &pt3);
     }
+
+    #[test]
+    fn cmp_around_fuzz_i8(pt1 in any_8(), pt2 in any_8(), pt3 in any_8()) {
+      let _ = pt1.ccw_cmp_around(&pt2, &pt3);
+    }
+
+    #[test]
+    fn bigint_colinear(pt1 in any_r(), pt2 in any_r()) {
+      let diff = &pt2 - &pt1;
+      let pt3 = &pt2 + &diff;
+      prop_assert!(Orientation::is_colinear(&pt1, &pt2, &pt3))
+    }
+
+    #[test]
+    fn bigint_not_colinear(pt1 in any_r(), pt2 in any_r()) {
+      let diff = &pt2 - &pt1;
+      let pt3 = &pt2 + &diff + &Vector([BigInt::from(1),BigInt::from(1)]);
+      prop_assert!(!Orientation::is_colinear(&pt1, &pt2, &pt3))
+    }
+
+    #[test]
+    fn orientation_reverse(pt1 in any_64(), pt2 in any_64(), pt3 in any_64()) {
+      let abc = Orientation::new(&pt1, &pt2, &pt3);
+      let cba = Orientation::new(&pt3, &pt2, &pt1);
+      prop_assert_eq!(abc, cba.reverse())
+    }
   }
 
   #[test]
   fn test_turns() {
     assert_eq!(
-      Point::new([0, 0]).orientation(&Point::new([1, 1]), &Point::new([2, 2])),
+      Orientation::new(
+        &Point::new([0, 0]),
+        &Point::new([1, 1]),
+        &Point::new([2, 2])
+      ),
       CoLinear
     );
     assert_eq!(
-      Point::new_nn([0.0, 0.0]).orientation(&Point::new_nn([1.0, 1.0]), &Point::new_nn([2.0, 2.0])),
+      Orientation::new(
+        &Point::new_nn([0.0, 0.0]),
+        &Point::new_nn([1.0, 1.0]),
+        &Point::new_nn([2.0, 2.0])
+      ),
       CoLinear
     );
 
@@ -378,4 +472,40 @@ pub mod tests {
       CoLinear
     );
   }
+
+  #[test]
+  fn unit_1() {
+    assert_eq!(
+      Point::new([0, 0]).orientation(&Point::new([1, 0]), &Point::new([1, 0])),
+      CoLinear
+    );
+    assert_eq!(
+      Point::new([0, 0]).orientation(&Point::new([1, 0]), &Point::new([2, 0])),
+      CoLinear
+    );
+    assert_eq!(
+      Point::new([1, 0]).orientation(&Point::new([2, 0]), &Point::new([0, 0])),
+      CoLinear
+    );
+    assert_eq!(
+      Point::new([1, 0]).orientation(&Point::new([2, 0]), &Point::new([1, 0])),
+      CoLinear
+    );
+  }
+
+  #[test]
+  fn unit_2() {
+    assert_eq!(
+      Point::new([1, 0]).orientation(&Point::new([0, 6]), &Point::new([0, 8])),
+      ClockWise
+    );
+  }
+
+  #[test]
+  fn unit_3() {
+    assert_eq!(
+      Point::new([-12_i8, -126]).orientation(&Point::new([-12, -126]), &Point::new([0, -126])),
+      CoLinear
+    );
+  }
 }
